/*
 * Universelle
 * LCD-Display Steuerung
 *
 * Copyright (c) 2010, Michael Hohl
 *
 */
 
#ifndef LCD_ROUTINES_H
#define LCD_ROUTINES_H
 
/*
 * Taktfrequenz
 * 
 */
#ifndef F_CPU
#warning F_CPU not set, for LCD driver!
#endif
 
/*
 * Pinbelegung
 * 
 */
 
//  LCD DB4-DB7 <-->  PORTD Bit PD0-PD3
#define LCD_PORT      PORTC
#define LCD_DDR       DDRC
#define LCD_DB        PC0
 
//  LCD RS      <-->  PORTD Bit PD4     (RS: 0=Data, 1=Command)
#define LCD_RS        PC4
 
//  LCD EN      <-->  PORTD Bit PD5     (EN: 1-Impuls fr Daten)
#define LCD_EN        PC5
 
/*
 * Ausfhrungszeiten
 * (ntig, wenn ohne Ready-Flag gearbeitet wird.)
 *
 * HINWEIS: Alle angaben in Millisekunden!
 * 
 */
 
#define LCD_BOOTUP_MS           35
#define LCD_ENABLE_US           1
#define LCD_WRITEDATA_US        46
#define LCD_COMMAND_US          42
 
#define LCD_SOFT_RESET_MS1      5
#define LCD_SOFT_RESET_MS2      1
#define LCD_SOFT_RESET_MS3      1
#define LCD_SET_4BITMODE_MS     5
 
#define LCD_CLEAR_DISPLAY_MS    2
#define LCD_CURSOR_HOME_MS      2
 
/* 
 * Zeilendefinition
 * (Positionen im RAM des LCD-Kontrollers)
 *
 */
 
#define LCD_DDADR_LINE1         0x00
#define LCD_DDADR_LINE2         0x40
#define LCD_DDADR_LINE3         0x10
#define LCD_DDADR_LINE4         0x50
 
/*
 * Initialisiert den LCD-Kontroller.
 *
 */
void lcd_init( void );
 
/*
 * Lscht den Bildschirminhalt des LCDs
 * und setzt den Cursor zurck auf HOME.
 *
 */
void lcd_clear( void );
 
/*
 * Setzt den Cursor auf HOME
 * (HOME = Ausgangsposition)
 *
 */
void lcd_home( void );
 
/*
 * Setzt den Cursor an eine beliebige Stelle
 *
 */
void lcd_setcursor( uint8_t spalte, uint8_t zeile );
 
/*
 * Sendet Daten an den LCD-Kontroller.
 *
 */
void lcd_data( uint8_t data );
 
/*
 * Schreibt einen String auf das Display.
 *
 */
void lcd_string( const char *data );
 
/*
 * Definition eines benutzerdefinierten Sonderzeichens.
 * data muss auf ein Array[5] mit den Spaltencodes des zu definierenden Zeichens
 * zeigen
 *
 */
void lcd_generatechar( uint8_t code, const uint8_t *data );
 
/*
 * Sendet einen rohen Befehl an den LCD-Kontroller.
 * 
 */
void lcd_command( uint8_t data );
 
 
/*
 * Befehlsatz des LCDs
 * (Nur verndern, falls nicht kompatibler LCD Kontroller!)
 *
 */
 
// Clear Display -------------- 0b00000001
#define LCD_CLEAR_DISPLAY       0x01
 
// Cursor Home ---------------- 0b0000001x
#define LCD_CURSOR_HOME         0x02
 
// Set Entry Mode ------------- 0b000001xx
#define LCD_SET_ENTRY           0x04
 
#define LCD_ENTRY_DECREASE      0x00
#define LCD_ENTRY_INCREASE      0x02
#define LCD_ENTRY_NOSHIFT       0x00
#define LCD_ENTRY_SHIFT         0x01
 
// Set Display ---------------- 0b00001xxx
#define LCD_SET_DISPLAY         0x08
 
#define LCD_DISPLAY_OFF         0x00
#define LCD_DISPLAY_ON          0x04
#define LCD_CURSOR_OFF          0x00
#define LCD_CURSOR_ON           0x02
#define LCD_BLINKING_OFF        0x00
#define LCD_BLINKING_ON         0x01
 
// Set Shift ------------------ 0b0001xxxx
#define LCD_SET_SHIFT           0x10
 
#define LCD_CURSOR_MOVE         0x00
#define LCD_DISPLAY_SHIFT       0x08
#define LCD_SHIFT_LEFT          0x00
#define LCD_SHIFT_RIGHT         0x01
 
// Set Function --------------- 0b001xxxxx
#define LCD_SET_FUNCTION        0x20
 
#define LCD_FUNCTION_4BIT       0x00
#define LCD_FUNCTION_8BIT       0x10
#define LCD_FUNCTION_1LINE      0x00
#define LCD_FUNCTION_2LINE      0x08
#define LCD_FUNCTION_5X7        0x00
#define LCD_FUNCTION_5X10       0x04
 
#define LCD_SOFT_RESET          0x30
 
// Set CG RAM Address --------- 0b01xxxxxx  (Character Generator RAM)
#define LCD_SET_CGADR           0x40
 
#define LCD_GC_CHAR0            0
#define LCD_GC_CHAR1            1
#define LCD_GC_CHAR2            2
#define LCD_GC_CHAR3            3
#define LCD_GC_CHAR4            4
#define LCD_GC_CHAR5            5
#define LCD_GC_CHAR6            6
#define LCD_GC_CHAR7            7
 
// Set DD RAM Address --------- 0b1xxxxxxx  (Display Data RAM)
#define LCD_SET_DDADR           0x80
 
#endif 
